 /*
Copyright 2002-2004 Steve Jolly

This file is part of Cont

Cont is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; specifically version 2 of the License.

Cont is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Cont; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

import org.jibble.pircbot.*;
import java.util.*;

public class Cont extends PircBot {

	public String ServerName;
	public int ServerPort;
	public String ServerPassword;
	public String ChannelName;
	public String BotName;
	private int DEFAULT_CONTEXT_LIST_LENGTH=10;
	private ContextList ContextList=new ContextList(DEFAULT_CONTEXT_LIST_LENGTH);
	private SupaVector UserList=new SupaVector();
 	private Hashtable LastSaw=new Hashtable();
	private CurrentTime Now=new CurrentTime();

	public static String DebugChannel = new String("#bot");

	public Cont(String CServerName, int CServerPort, String CServerPassword, String CChannelName, String CBotName) {
		ServerName=CServerName;
		ServerPort=CServerPort;
		ServerPassword=CServerPassword;
		ChannelName=CChannelName;
		BotName=CBotName;
		this.setName(BotName);
		this.setLogin(BotName);
		UserList.read("userlist.dat");
	}

	public void onMessage(String channel, String sender, String login, String hostname, String message) {
		IRCLine m = new IRCLine(message);
		m.checkAddressed(BotName);
		messageHandler(channel, sender, login, hostname, m);
		ContextList.add(Now.now()+" <"+sender+"> "+message);
	}

	public void onPrivateMessage(String sender, String login, String hostname, String message) {
		IRCLine m = new IRCLine(message);
		m.checkAddressed(BotName);
		m.addressed = true;	//it's a private message, right?
		messageHandler(sender, sender, login, hostname, m);
	}

	private void messageHandler(String channel, String sender, String login, String hostname, IRCLine m) {
		if (m.addressed) {
			if (m.starts("enable context")) {
				m.remove(0);
				if (m.size()>1) {
					while (m.size()>1) {
						m.remove(0);
						addPair(sender, m.elementAt(0).toString());
					}
				} else if (sender.equals(channel)) {
					try {
						addPair(sender, getChannels()[0]);
					} catch (Exception e) {
						sendMessage(sender, "Error adding you: couldn't work out what channel you meant.");
					}
				} else {
					addPair(sender, channel);
				}
			}
			else if (m.starts("disable context")) {
				m.remove(0);
				if (m.size()>1) {
					while (m.size()>1) {
						m.remove(0);
						removePair(sender, m.elementAt(0).toString());
					}
				} else if (sender.equals(channel)){
					try {
						removePair(sender, getChannels()[0]);
					} catch (Exception e) {
						sendMessage(sender, "Error removing you: couldn't work out what channel you meant.");
					}
				} else {
					removePair(sender,channel);
				}
			}
			else if (m.starts("instacontext")) {
				String targetChannel;
				if ((m.size()==2)&&(validChannel(m.elementAt(1).toString()))) {
					targetChannel=m.elementAt(1).toString();
				} else {
					targetChannel=channel;
				}
				sendContext(sender, targetChannel);
			}
			else if (m.starts("help")) {
				m.remove(0);
				if (m.size()==0) {
					sendMessage(channel, sender+": syntax is '"+BotName+": command. Available commands are enable context, disable context and instacontext. Try 'help command' for details");
				} else if (m.starts("enable context")) {
					sendMessage(channel, sender+": 'enable context [for #channel]' adds you to the list of people who will receive context upon joining #channel.");
				} else if (m.starts("disable context")) {
					sendMessage(channel, sender+": 'disable context [for #channel]' removes you from the list of people who will receive context upon joining #channel.");
				} else if (m.starts("instacontext")) {
					sendMessage(channel, sender+": 'instacontext [#channel]' gives you instant context. This is mostly useful for testing.");
				} else {
					sendMessage(channel, sender+": sorry - that command is not recognised");
				}
			}
		}
	}

	public void onAction(String sender, String login, String hostname, String target, String action) {
		ContextList.add(Now.now()+" * "+sender+" "+action);
	}

	protected void onJoin(String channel, String sender, String login, String hostname) {
		if (UserList.contains(sender+channel)) sendContext(sender, channel);
		ContextList.add(Now.now()+" *** "+sender+"("+login+"@"+hostname+") has joined "+channel);
	}

	protected void onDisconnect () {
		int tries=0;
		while (tries<1000 && !isConnected()) {
			try {
				connect(ServerName, ServerPort, ServerPassword);
			} catch (Exception e) {
				System.out.println("Connection attempt "+tries+" failed.");
				tries++;
				if (tries>=1000000) System.exit(0);
			}
		} //while
		joinChannel(ChannelName);
		sendAction(ChannelName, "poings");
	}

	protected void onKick (String channel, String kickerNick, String kickerLogin, String kickerHostname, String recipientNick, String reason) {
		String KickMessage=Now.now()+" *** "+recipientNick+" was kicked by "+kickerNick+" ("+reason+")";
		if (recipientNick.equals(BotName)) {
			joinChannel(channel);
		} else {
			LastSaw.remove(recipientNick+channel);
			LastSaw.put(recipientNick+channel,KickMessage);
		}
		ContextList.add(KickMessage);
	}

	protected void onNickChange(String oldNick, String login, String hostname, String newNick) {
		ContextList.add(Now.now()+" *** "+oldNick+" ("+login+"@"+hostname+") is now known as "+newNick);
	}

	protected void onPart(String channel, String sender, String login, String hostname) {
		String PartMessage=Now.now()+" *** "+sender+" ("+login+"@"+hostname+") has left "+channel;
		if (!sender.equals(BotName)) {
			LastSaw.remove(sender+channel);
			LastSaw.put(sender+channel,PartMessage);
		}
		ContextList.add(PartMessage);
	}

	protected void onMode(String channel, String sourceNick, String sourceLogin, String sourceHostname, String mode) {
		ContextList.add(Now.now()+" *** "+sourceNick+" ("+sourceLogin+"@"+sourceHostname+") sets mode "+mode);
	}

	protected void onQuit(String sourceNick, String sourceLogin, String sourceHostname, String reason) {
		String QuitMessage=new String("*** "+sourceNick+" ("+sourceLogin+"@"+sourceHostname+") has quit IRC ("+reason+")");
		if (!sourceNick.equals(BotName)) {
			List ChannelList=Arrays.asList(getChannels());
			ListIterator It = ChannelList.listIterator();
			while (It.hasNext()) {
				String NextChannel = It.next().toString();
				LastSaw.remove(sourceNick+NextChannel);
				LastSaw.put(sourceNick+NextChannel,QuitMessage);	//not a perfect solution
			}
		}
		ContextList.add(Now.now()+" "+QuitMessage);
	}

	//a channel is valid if it conforms to the IRC RFC naming requirements
	//and is also a channel that the bot is on.  (Yes, it's redundant... kinda)
	private boolean validChannel(String channel) {
		if (channel.matches("(?:#|&)[^ \\a\\x00\\r\\n,]+")) {
			if (Arrays.asList(getChannels()).contains(channel)) return true;
		}
		return false;
	}

	//add a user/channel pair to the UserList.
	private void addPair(String user, String channel) {
		if (validChannel(channel)&&(!UserList.contains(user+channel))) {
			UserList.add(user+channel);
			UserList.write("userlist.dat");
			sendMessage(user,"Enabling context for you on channel "+channel);
		}
	}

	private void removePair(String user, String channel) {
		if (UserList.contains(user+channel)) {
			UserList.remove(user+channel);
			UserList.write("userlist.dat");
			sendMessage(user,"Disabling context for you on channel "+channel);
		}
	}

	private void sendContext(String recipient, String channel) {
		ListIterator IUsers=ContextList.listIterator(0);
		if (LastSaw.containsKey(recipient+channel)) {
System.out.println(LastSaw.get(recipient+channel));
			if (ContextList.contains(LastSaw.get(recipient+channel))) {
				while ((IUsers.hasNext())&&(!IUsers.next().equals(LastSaw.get(recipient+channel)))) {
					;
				}
			}
		}
		while (IUsers.hasNext()) {
			String zzz=IUsers.next().toString();		
			sendMessage(recipient, zzz);
		}
	}
}

